"""
Django settings for kelvpn_backend project.

Generated by 'django-admin startproject' using Django 5.1.1.

For more information on this file, see
https://docs.djangoproject.com/en/5.1/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/5.1/ref/settings/
"""

from pathlib import Path
from datetime import timedelta
import os
from datetime import datetime
import logging.handlers

# Build paths inside the project like this: BASE_DIR / 'subdir'.
BASE_DIR = Path(__file__).resolve().parent.parent


# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/5.1/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = "django-insecure-%(nb(42q5#p)$+yli0p7fwd4!uo5g_dskq+lf+8kzo-#qi8-z-"

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

ALLOWED_HOSTS = ["*"]


# Application definition

INSTALLED_APPS = [
    "corsheaders",
    "channels",
    "api",
    "ninja",
    "rest_framework_simplejwt.token_blacklist",
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django.contrib.staticfiles",
]

MIDDLEWARE = [
    "corsheaders.middleware.CorsMiddleware",
    "django.middleware.security.SecurityMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
]

ROOT_URLCONF = "kelvpn_backend.urls"

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
            ],
        },
    },
]

WSGI_APPLICATION = "kelvpn_backend.wsgi.application"
ASGI_APPLICATION = "kelvpn_backend.asgi.application"

CHANNEL_LAYERS = {
    "default": {
        "BACKEND": "channels.layers.InMemoryChannelLayer",
    },
}

# Database
# https://docs.djangoproject.com/en/5.1/ref/settings/#databases

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": BASE_DIR / "db.sqlite3",
        "OPTIONS": {
               "timeout": 30,
           },
    }
}


# Password validation
# https://docs.djangoproject.com/en/5.1/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        "NAME": "django.contrib.auth.password_validation.UserAttributeSimilarityValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.MinimumLengthValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.CommonPasswordValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.NumericPasswordValidator",
    },
]


# Internationalization
# https://docs.djangoproject.com/en/5.1/topics/i18n/

LANGUAGE_CODE = "en-us"

TIME_ZONE = "UTC"

USE_I18N = True

USE_TZ = True


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/5.1/howto/static-files/

STATIC_URL = "static/"

# Default primary key field type
# https://docs.djangoproject.com/en/5.1/ref/settings/#default-auto-field

DEFAULT_AUTO_FIELD = "django.db.models.BigAutoField"

# Custom handler that creates files with date prefix and auto-creates directories
class DatePrefixTimedRotatingFileHandler(logging.handlers.TimedRotatingFileHandler):
    def __init__(self, filename, when='midnight', interval=1, backupCount=30, encoding=None, delay=False, utc=False, atTime=None):
        self.base_filename = filename
        # Ensure directory exists before calling parent constructor
        self._ensure_directory_exists(filename)
        super().__init__(filename, when, interval, backupCount, encoding, delay, utc, atTime)
    
    def _ensure_directory_exists(self, filename):
        """Ensure the directory for the log file exists and has proper permissions"""
        import os
        import stat
        
        dirname = os.path.dirname(filename)
        if dirname and not os.path.exists(dirname):
            try:
                os.makedirs(dirname, mode=0o755, exist_ok=True)
                # Try to set ownership to kelvpn:www-data if running as root
                try:
                    import pwd
                    import grp
                    kelvpn_uid = pwd.getpwnam('kelvpn').pw_uid
                    www_data_gid = grp.getgrnam('www-data').gr_gid
                    os.chown(dirname, kelvpn_uid, www_data_gid)
                except (KeyError, PermissionError, OSError):
                    # If we can't set ownership, that's okay - continue anyway
                    pass
            except OSError as e:
                # If we can't create directory, fall back to console logging
                print(f"Warning: Could not create log directory {dirname}: {e}")
    
    def _open(self):
        """Override _open to ensure file exists and has proper permissions before opening"""
        # Ensure directory still exists (in case it was deleted)
        self._ensure_directory_exists(self.baseFilename)
        
        # Ensure the log file exists
        if not os.path.exists(self.baseFilename):
            try:
                # Create the file
                open(self.baseFilename, 'a').close()
                # Set proper permissions
                os.chmod(self.baseFilename, 0o644)
                # Try to set ownership
                try:
                    import pwd
                    import grp
                    kelvpn_uid = pwd.getpwnam('kelvpn').pw_uid
                    www_data_gid = grp.getgrnam('www-data').gr_gid
                    os.chown(self.baseFilename, kelvpn_uid, www_data_gid)
                except (KeyError, PermissionError, OSError):
                    pass
            except OSError as e:
                print(f"Warning: Could not create log file {self.baseFilename}: {e}")
        
        return super()._open()
    
    def rotation_filename(self, default_name):
        # Extract directory and base filename
        dirname = os.path.dirname(self.base_filename)
        basename = os.path.basename(self.base_filename)
        
        # Get yesterday's date
        from datetime import datetime, timedelta
        yesterday = datetime.now() - timedelta(days=1)
        date_str = yesterday.strftime('%Y-%m-%d')
        
        # Create filename with date prefix
        return os.path.join(dirname, f"{date_str}_{basename}")

# Function to get safe handlers list with fallback
def get_safe_handlers():
    """Get list of handlers with fallback to console if file handler fails"""
    try:
        # Try to create a test log entry to verify file handler works
        test_handler = DatePrefixTimedRotatingFileHandler(
            "/var/log/kelvpn-web-admin/kelvpn-web-admin.log"
        )
        test_handler.close()
        return ["console", "file"]
    except Exception as e:
        print(f"Warning: Primary log file handler failed: {e}. Using fallback.")
        return ["console", "fallback_file"]

LOGGING = {
    "version": 1,
    "disable_existing_loggers": False,
    "handlers": {
        "console": {
            "class": "logging.StreamHandler",
        },
        "file": {
            "()": DatePrefixTimedRotatingFileHandler,
            "filename": "/var/log/kelvpn-web-admin/kelvpn-web-admin.log",
            "when": "midnight",
            "interval": 1,
            "backupCount": 30,
            "encoding": "utf-8",
            "level": "DEBUG",
            "formatter": "verbose",
        },
        "fallback_file": {
            "class": "logging.FileHandler",
            "filename": "/tmp/kelvpn-web-admin-fallback.log",
            "encoding": "utf-8",
            "level": "DEBUG",
            "formatter": "verbose",
        },
    },
    "formatters": {
        "verbose": {
            "format": "%(asctime)s [%(levelname)s] %(name)s: %(message)s"
        },
    },
    "root": {
        "handlers": get_safe_handlers(),
        "level": "DEBUG",
    },
    "loggers": {
        "django": {
            "handlers": get_safe_handlers(),
            "level": "INFO",
            "propagate": True,
        },
        "kelvpn_backend": {
            "handlers": get_safe_handlers(),
            "level": "DEBUG",
            "propagate": False,
        },
        "api": {
            "handlers": get_safe_handlers(),
            "level": "DEBUG",
            "propagate": False,
        },
        "api.services": {
            "handlers": get_safe_handlers(),
            "level": "DEBUG",
            "propagate": False,
        },
    },
}

CORS_ALLOW_ALL_ORIGINS = True

SIMPLE_JWT = {
    'ACCESS_TOKEN_LIFETIME': timedelta(minutes=60),
    'REFRESH_TOKEN_LIFETIME': timedelta(days=1),
    'ROTATE_REFRESH_TOKENS': False,
    'BLACKLIST_AFTER_ROTATION': True,
    'ALGORITHM': 'HS256',
    'SIGNING_KEY': SECRET_KEY,
    'AUTH_HEADER_TYPES': ('Bearer',),
}

HOSTAPD_CONFIG_PATHS = {
    "wlan0": "/etc/hostapd/wlan0.conf",
    "wlan1": "/etc/hostapd/wlan1.conf",
}
